\name{ltmle}
\alias{ltmle}
\alias{ltmleMSM}
\title{
Longitudinal Targeted Maximum Likelihood Estimation
}
\description{
\code{ltmle} is Targeted Maximum Likelihood Estimation (TMLE) of treatment/censoring specific mean outcome for point-treatment and longitudinal data. \code{ltmleMSM} adds Marginal Structural Models. Both always provide Inverse Probability of Treatment/Censoring Weighted estimate (IPTW) as well. Maximum likelihood based G-computation estimate (G-comp) can be obtained instead of TMLE. \code{ltmle} can be used to calculate additive treatment effect, risk ratio, and odds ratio.
}
\usage{
ltmle(data, Anodes, Cnodes=NULL, Lnodes=NULL, Ynodes, Qform=NULL, gform=NULL, abar, gbounds=c(0.01, 1), deterministic.acnode.map=NULL, stratify=FALSE, SL.library=NULL, estimate.time=nrow(data) > 50, gcomp=FALSE)
ltmleMSM(data, Anodes, Cnodes=NULL, Lnodes=NULL, Ynodes, Qform=NULL, gform=NULL, gbounds=c(0.01, 1), deterministic.acnode.map=NULL, SL.library=NULL, regimens, working.msm, summary.measures, summary.baseline.covariates=NULL, final.Ynodes=NULL, pooledMSM=T, stratify=F, weight.msm=T, estimate.time=nrow(data) > 50, gcomp=FALSE)
}
\arguments{
  \item{data}{data frame following the time-ordering of the nodes. See 'Details'.}
  \item{Anodes}{column names or indicies in \code{data} of treatment nodes}
  \item{Cnodes}{column names or indicies in \code{data} of censoring nodes}
  \item{Lnodes}{column names or indicies in \code{data} of time-dependent covariate nodes}
  \item{Ynodes}{column names or indicies in \code{data} of outcome nodes}
  \item{Qform}{character vector of regression formulas for \eqn{Q}. See 'Details'.}
  \item{gform}{character vector of regression formulas for \eqn{g}. See 'Details'.}
  \item{abar}{binary vector (numAnodes x 1) or matrix (n x numAnodes) of counterfactual treatment}
  \item{gbounds}{lower and upper bounds on estimated probabilities for g-factors. Vector of length 2, order unimportant.}
  \item{deterministic.acnode.map}{optional information on A and C nodes that are given deterministically. See 'Details'. Default \code{NULL} indicates no deterministic links}
  \item{stratify}{if \code{TRUE} stratify on following abar when estimating Q and g. If \code{FALSE}, pool over \code{abar}.}
  \item{SL.library}{optional character vector of libraries to pass to \code{\link[SuperLearner:SuperLearner]{SuperLearner}}. \code{NULL} indicates \link{glm} should be called instead of \code{\link[SuperLearner:SuperLearner]{SuperLearner}}. '\code{default}' indicates a standard set of libraries. See 'Details'.}
  \item{estimate.time}{if \code{TRUE}, run an initial estimate using only 50 observations and use this to print a very rough estimate of the total time to completion. No action if there are fewer than 50 observations.}
  \item{gcomp}{if \code{TRUE}, run the maximum likelihood based G-computation estimate \emph{instead} of TMLE}
  \item{regimens}{binary array: n x numAnodes x numRegimens of counterfactual treatment}
  \item{working.msm}{character formula for the working marginal structural model}
  \item{summary.measures}{array: num.regimens x numSummaryMeasures x num.final.Ynodes - measures summarizing the regimens that will be used on the right hand side of working.msm}
  \item{summary.baseline.covariates}{NOT FULLY IMPLEMENTED YET - leave as NULL (default)}
  \item{final.Ynodes}{vector subset of Ynodes - used in MSM to pool over a set of outcome nodes}
  \item{pooledMSM}{if \code{TRUE}, the TMLE targeted step will pool across regimens}
  \item{weight.msm}{if \code{TRUE}, the working.msm will be weighted by the empirical probability of each regimen [in the future more flexible weighting may be possible]} 
}
\details{
  The estimates returned by \code{ltmle} are of a treatment specific mean, \eqn{E[Y_{\bar{a}}]}, the mean of the final treatment node, where all treatment nodes, \eqn{A}, are set to \eqn{\bar{a}} (\code{abar}) and all censoring nodes \eqn{C} are set to 1 (uncensored). The estimates returned by \code{ltmleMSM} are similar but are the parameters in a working marginal structural model.
  
  By calling \code{ltmle} twice, using two different values of \code{abar}, additive treatment effect, risk ratio, and odds ratio can be computed using \code{\link{summary.ltmle}}. 
  
  \code{data} should be a data frame where the order of the columns corresponds to the time-ordering of the model. 
  \itemize{
      \item in censoring columns (Cnodes): 1 = uncensored, 0 = censored (must be binary)
      \item in treatment columns (Anodes): 1 = treated, 0 = untreated (must be binary)
      \item in event columns (Ynodes): 1 = event (e.g. death), 0 = no event (must be binary)
      \item time-dependent covariate columns (Lnodes): can be any numeric data
  }
  
  There may be columns in \code{data} that are not in any of \code{Cnodes}, \code{Anodes}, \code{Ynodes}, and \code{Lnodes}.
  
  The events in Ynodes must be of the form where once Y jumps to 1, Y remains 1 at subsequent nodes. Future releases of this package may relax this assumption. 
  
  A node should only be classified as an Lnode if it comes after a Cnode or Anode. If it does not, it is considered a a baseline covariate, not a time-dependent covariate. 
  
  \code{Qform} should be \code{NULL}, in which case all parent nodes of each L and Y node will be used as regressors, or a named character vector that can be coerced to class "\code{\link{formula}}". The length of \code{Qform} must be equal to \code{length(Lnodes) + length(Ynodes)} and the names and order of the formulas must be the same as the names and order of the L and Y nodes in \code{data}. The left hand side of each formula should be "\code{Q.kplus1}". If \code{SL.library} is \code{NULL}, \code{glm} will be called using the elements of \code{Qform}. If \code{SL.library} is specified, \code{\link[SuperLearner:SuperLearner]{SuperLearner}} will be called and all variables appearing on the right hand side of a formula will be passed to \code{\link[SuperLearner:SuperLearner]{SuperLearner}}. The actual functional form of the formula is unimportant if \code{\link[SuperLearner:SuperLearner]{SuperLearner}} is called.
  
  \code{gform} should be \code{NULL}, in which case all parent nodes of each L and Y node will be used as regressors, or a character vector that can be coerced to class "\code{\link{formula}}". The length of \code{gform} must be equal to \code{length(Anodes) + length(Cnodes)} and the order of the formulas must be the same as the order the A and C nodes appear in \code{data}. The left hand side of each formula should be the name of the Anode or Cnode. If \code{SL.library} is \code{NULL}, \code{glm} will be called using the elements of \code{gform}. If \code{SL.library} is specified, \code{\link[SuperLearner:SuperLearner]{SuperLearner}} will be called and all variables appearing on the right hand side of a formula will be passed to \code{\link[SuperLearner:SuperLearner]{SuperLearner}}. The actual functional form of the formula is unimportant if \code{\link[SuperLearner:SuperLearner]{SuperLearner}} is called.
  
  \code{abar} specifies the counterfactual values of the Anodes, using the order they appear in \code{data} and should have the same length (if abar is a vector) or number of columns (if abar is a matrix) as \code{Anodes}.
  
  \code{deterministic.acnode.map} can be used to specify model knowledge about value of Anodes and/or Cnodes that are set deterministically. For example, it may be the case that once a patient starts treatment, they always stay on treatment. \code{deterministic.acnode.map} is a list [1 ... number of deterministic nodes] of lists with the following components:
    \itemize{
      \item \code{node} - the name or index of the node 
      \item \code{is.deterministic} - logical vector, length equal to \code{nrow(data)}, \code{TRUE} if a patient's status for this node is deterministic
      \item \code{prob1} - probability that node is 1 (treated/uncensored), length equal to 1 or \code{length(which(is.deterministic))}
}

If \code{SL.library} is the string '\code{default}', \code{SL.library} is set to \code{list("SL.glm", "SL.glmnet", "SL.stepAIC", "SL.bayesglm", c("SL.glm", "screen.corP"), c("SL.glmnet", "screen.corP"), c("SL.step", "screen.corP"), c("SL.step.forward", "screen.corP"), c("SL.stepAIC", "screen.corP"), c("SL.step.interaction", "screen.corP"), c("SL.bayesglm", "screen.corP"))}. Note that the default set of libraries consists of main terms models. It may be advisable to include squared terms, interaction terms, etc in \code{data} or include libraries that consider non-linear terms.

The print method for \code{ltmle} objects only prints the tmle estimates. 
}
\value{
\code{ltmle} returns an object of class "\code{ltmle}"
The function \code{\link{summary}} (i.e. \code{\link{summary.ltmle}}) can be used to obtain or print a summary of the results.
An object of class "\code{ltmle}" is a list containing the following components:
\item{estimates}{a named vector of length 4 with elements, each an estimate of \eqn{E[Y_{bar{a}}]}:
  \itemize{
  \item \code{tmle} - Targeted Maximum Likelihood Estimate [NULL if \code{gcomp} is \code{TRUE}]
  \item \code{iptw} - Inverse Probability of Treatment/Censoring Weighted estimate 
  \item \code{gcomp} - maximum likelihood based G-computation estimate [NULL if \code{gcomp} is \code{FALSE}]
  \item \code{naive} - naive estimate \eqn{E[Y|a=\bar{a}]}
  }
}
\item{regression.stats}{a list with the following components, useful for diagnostics
  \itemize{
  \item \code{g} - matrix, 3 x number of A,C nodes; \code{observations} - number of observations; \code{events} - number of events; \code{eventsPerObs}
  \item \code{Q} - matrix, 3 x number of L,Y nodes; \code{observations} - number of observations; \code{events} - sum of Q.kplus1; \code{eventsPerObs}
  \item \code{min.cum.g} - minimum value of cumulative g, before bounding
  }
}
\item{IC}{a list with the following components of Influence Curve values}
  \itemize{
  \item \code{tmle} - vector of influence curve values for Targeted Maximum Likelihood Estimate [NULL if \code{gcomp} is \code{TRUE}]
  \item \code{iptw} - vector of influence curve values for Inverse Probability of Treatment/Censoring Weighted estimate 
  \item \code{gcomp} - vector of influence curve values for Targeted Maximum Likelihood Estimate without updating [NULL if \code{gcomp} is \code{FALSE}]
  }
\item{call}{the matched call}
\item{gcomp}{the \code{gcomp} input}

\code{ltmleMSM} returns an object of class "\code{ltmleMSM}"
The function \code{\link{summary}} (i.e. \code{\link{summary.ltmleMSM}}) can be used to obtain or print a summary of the results.
An object of class "\code{ltmleMSM}" is a list containing the following components:
\item{beta}{parameter estimates for working.msm using TMLE (GCOMP if \code{gcomp} input is \code{TRUE})}
\item{beta.iptw}{parameter estimates for working.msm using IPTW (\code{NULL} if \code{pooledMSM} is \code{TRUE})}
\item{IC}{matrix, n x numBetas - influence curve values for TMLE (without updating if \code{gcomp} input is \code{TRUE})}
\item{IC.iptw}{matrix, n x numBetas - influence curve values for IPTW (\code{NULL} if \code{pooledMSM} is \code{TRUE})}
\item{msm}{object of class glm - the result of fitting the working.msm}
\item{cum.g}{matrix, n x numACnodes - cumulative g, after bounding}
\item{call}{the matched call}
\item{gcomp}{the \code{gcomp} input}

Note: I may try to make the outputs to ltmle and ltmleMSM match up better in the public version.
}

\references{
van der Laan, Mark J. and Gruber, Susan, "Targeted Minimum Loss Based Estimation of an Intervention Specific Mean Outcome" (August 2011). U.C. Berkeley Division of Biostatistics Working Paper Series. Working Paper 290.
\url{http://biostats.bepress.com/ucbbiostat/paper290}

van der Laan, Mark J. and Rose, Sherri, "Targeted Learning: Causal Inference for Observational and Experimental Data" New York: Springer, 2011.
}
\author{
Joshua Schwab \email{joshuaschwab@yahoo.com}, based on the work of Susan Gruber and Mark van der Laan
}

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
\code{\link{summary.ltmle}}, \code{\link{summary.ltmleMSM}}, \code{\link[SuperLearner:SuperLearner]{SuperLearner}}
}
\examples{
rexpit <- function(x) rbinom(n=length(x), size=1, prob=plogis(x))
                 
# Example 1: Single time point example. True value of E[Y_1] (expected value of
#   Y setting A to 1) is approximately 0.5939.
set.seed(2)
n <- 1000
W1 <- rnorm(n)
W2 <- rbinom(n, size=1, prob=0.3)   
W3 <- rnorm(n)
A <- rexpit(-1 + 2 * W1^2)
Y <- rexpit(-0.5 + 2 * W1^2 + 0.5 * W2 - 0.5 * A + 0.2 * W3 * A 
       - 1.1 * W3 + 0.2 * rnorm(n))

data <- data.frame(W1, W2, W3, A, Y)

library(SuperLearner)

#SuperLearner semiparametric estimation using all parents as regressors 
result1 <- ltmle(data, Anodes="A", Lnodes=NULL, Ynodes="Y", abar=1, 
  SL.library=c("SL.glm", "SL.step", "SL.mean"))
summary(result1)
summary(result1, estimator="iptw")

#SuperLearner semiparametric estimation using (incorrectly) specified regressors
#note: The functional form for Qform and gform is unimportant if 
# using SuperLearner - see 'Details'
result2 <- ltmle(data, Anodes="A", Lnodes=NULL, Ynodes="Y", 
 Qform=c(Y="Q.kplus1 ~ W1 + W3 + A"), gform="A ~ W1", abar=1, 
 SL.library=c("SL.glm", "SL.step", "SL.mean"))

#glm using correctly specified Qform and gform
result3.abar1 <- ltmle(data, Anodes="A", Lnodes=NULL, Ynodes="Y", 
 Qform=c(Y="Q.kplus1 ~ I(W1^2) + W2 + W3*A"), gform="A ~ I(W1^2)", 
 abar=1, SL.library=NULL)


#Get summary measures (additive treatment effect, odds ratio, relative risk) 
#  for abar=1 vs abar=0
result3.abar0 <- ltmle(data, Anodes="A", Lnodes=NULL, Ynodes="Y", 
 Qform=c(Y="Q.kplus1 ~ I(W1^2) + W2 + W3*A"), gform="A ~ I(W1^2)", 
 abar=0, SL.library=NULL)
summary(result3.abar1, result3.abar0)


# Example 2: Longitudinal example. Includes informative censoring and treatment. 
# Time ordering of data is W, C1, L1, A1, Y1, C2, L2, A2, Y2
# True value of E[Y_(1,1,1,1)] (expected value of Y setting C1, A1, C2, A2 all to 1)
#  is approximately 0.413.
# A1 is known to always be 1 if L1 < -2, and is 1 with probability 0.1 if L1 > 2 
# A2 is known to always be 1 if A1 is 1 
# We incorporate this knowledge using deterministic.acnode.map

# Generate data:
ua <- rep(TRUE, n)   #ua = uncensored and alive
L1 <- A1 <- Y1 <- C2 <- L2 <- A2 <- Y2 <- as.numeric(rep(NA, n))
W <- rnorm(n)
C1 <- rexpit(2 + W)
ua <- ua & C1
L1[ua] <- rnorm(n)[ua] + W[ua]
A1[ua] <- rexpit(L1[ua])
A1[ua & L1 < -2] <- 1
A1[ua & L1 >  2] <- rbinom(n, size=1, prob=0.1)[ua & L1 >  2]
Y1[ua] <- rexpit((W + L1 - A1)[ua])
ua <- ua & !Y1
C2[ua] <- rexpit((1 + 0.7 * L1 - A1)[ua])
ua <- ua & C2
L2[ua] <- (0.5 * L1 - 0.9 * A1 + rnorm(n))[ua]
A2[ua] <- rexpit((0.5 * L1 + 0.8 * L2)[ua]) | A1[ua]
Y2[ua] <- rexpit((0.7 * L1 + L2 - 0.8 * A1 - A2)[ua])
Y2[Y1 == 1] <- 1  # if a patient dies at time 1, record death at time 2 as well
data <- data.frame(W, C1, L1, A1, Y1, C2, L2, A2, Y2)

# Analyze data:
a1.det <- (L1 < -2 | L1 > 2) & C1 == 1
a1.prob1 <- ((L1 < -2) * 1 + (L1 > 2) * 0.1)[a1.det]
deterministic.acnode.map <- list(
 list(node="A1", is.deterministic=a1.det, prob1=a1.prob1), 
 list(node="A2", is.deterministic=A1==1 & !is.na(A2), prob1=1)) 

result4 <- ltmle(data, Anodes=c("A1","A2"), Cnodes=c("C1", "C2"), 
 Lnodes=c("L1", "L2"), Ynodes=c("Y1", "Y2"), abar=c(1, 1), 
 deterministic.acnode.map=deterministic.acnode.map, SL.library=NULL)
 
# Example 3: Dynamic treatment
# W -> A1 -> L -> A2 -> Y
# Treatment regimen of interest is: Always treat at time 1 (A1 = 1), treat at at time 2 (A2 = 1), iff L > 0
# True value of E[Y_d] is approximately 0.346

n <- 10000
W <- rnorm(n)
A1 <- rexpit(W)
L <- 0.3 * W + 0.2 * A1 + rnorm(n)
A2 <- rexpit(W + A1 + L)
Y <- rexpit(W - 0.6 * A1 + L - 0.8 * A2)
data <- data.frame(W, A1, L, A2, Y)

abar <- matrix(nrow=n, ncol=2)
abar[, 1] <- 1
abar[, 2] <- L > 0

result5 <- ltmle(data, Anodes=c("A1", "A2"), Lnodes="L", Ynodes="Y", abar=abar, SL.library=NULL)

# Example 4: MSM
# Given data over 5 time points where A switches to 1 once and stays 1. We want to know
# how death varies as a function of time and an indicator of whether a patient's intended
# regimen was to switch before time. For some reason, we are only interested in events from 
# times 3, 4, and 5.
data(sampleDataForLtmleMSM)
Anodes <- grep("^A", names(sampleDataForLtmleMSM$data))
Ynodes <- grep("^Y", names(sampleDataForLtmleMSM$data))
result6 <- ltmleMSM(sampleDataForLtmleMSM$data, Anodes=Anodes, Ynodes=Ynodes, regimens=sampleDataForLtmleMSM$regimens, 
 summary.measures=sampleDataForLtmleMSM$summary.measures[,,3:5], final.Ynodes=Ynodes[3:5], 
 working.msm="Y ~ time + I(switch.time <= time)")
print(summary(result6))

}
